@@ -8,8 +8,9 @@ from subprocess import Popen, PIPE, STDOUT
 from urllib.error import HTTPError
 
 
-from indico_install.utils import options_wrapper, run_cmd
+from indico_install.utils import options_wrapper, run_cmd, convertb64
 from indico_install.infra.input_utils import download_indicoapi_data, auth_with_gsutil
+from indico_install.kube.svc.restart import restart
 from indico_install.infra.single_node import single_node
 from indico_install.infra.gke import gke
 from indico_install.infra.aks import aks
@@ -40,9 +41,14 @@ def infra(ctx):
 
 @infra.command("generate-saml")
 @click.pass_context
-@click.argument("host", required=True, type=str)
-@click.argument("metadata_url", required=True, type=str)
-def generate_saml(ctx, host, metadata_url):
+@click.argument("host", required=False, type=str)
+@click.argument("metadata_url", required=False, type=str)
+@click.option(
+    "-f",
+    "--custom-saml",
+    help="Optional path to a saml settings.json file; if specified, will upload the given file as a noct secret",
+)
+def generate_saml(ctx, host=None, metadata_url=None, custom_saml=None):
     """
     Generate and inject SAML settings from Azure AD Saml2
     Before running, complete the following steps:
@@ -55,11 +61,32 @@ def generate_saml(ctx, host, metadata_url):
     Example usage:
     indico infra generate-saml app.indico.io https://login.microsoftonline.com/12346-78910-11121314/federationmetadata/2007-06/federationmetadata.xml
 
+    Alternatively, inject a custom settings.json file:
+    indico infra generate-saml -f custom-settings.json
+
     """
+    if custom_saml:
+        with open(custom_saml) as settings:
+            settings_content = json.load(settings)
+        run_cmd(
+            f'''kubectl create secret generic indico-sso-secrets --from-literal=settings.json="{settings_content}"'''
+        )
+        ctx.invoke(
+            restart, services=["noct"], contains=False, wait="2m"
+        )
+        return
+    
+    # not using custom settings
+    if not (host and metadata_url):
+        click.secho("Please provide a host and metadata url if not using a custom settings.json file", fg="red")
+        return
     try:
         obj = untangle.parse(metadata_url)
     except HTTPError:
-        click.secho("Federation URL not found. Please check that this is the correct URL", fg="red")
+        click.secho(
+            "Federation URL not found. Please check that this is the correct URL",
+            fg="red",
+        )
         return
     svc = obj.EntityDescriptor.IDPSSODescriptor.SingleSignOnService[0]["Location"]
     endpoint = obj.EntityDescriptor["entityID"]
@@ -99,8 +126,16 @@ def generate_saml(ctx, host, metadata_url):
         "certFingerprint": fin,
     }
     stg = {"strict": "true", "debug": "true", "sp": jsp, "idp": jid}
-    settings_content = json.dumps(stg, indent=4)
-    run_cmd(f"""kubectl create secret generic indico-sso-secrets --from-literal=settings.json='{settings_content}'""")
+    settings_content = convertb64(json.dumps(stg, indent=4))
+    run_cmd(
+        """kubectl patch secret indico-sso-secrets -p '{"data":{"settings.json":"""
+        f'"{settings_content}"'
+        "}}'"
+    )
+    ctx.invoke(
+        restart, services=["noct"], contains=False, wait="2m"
+    )
+
 
 @infra.command("download-apidata")
 @click.pass_context
